#include "freq_meter.h"
#include "nrfx_gpiote.h"
#include "nrfx_ppi.h"
#include <version.h>

#define TILT_PIN NRF_DT_GPIOTE_INST(DT_NODELABEL(tilt_int1), gpios)
#define IN_PIN NRF_DT_GPIOTE_INST(DT_NODELABEL(in_int1), gpios)

#if (KERNEL_VERSION_MAJOR == 3) && (KERNEL_VERSION_MINOR >= 5)
#define GPIOTE_INST	NRF_DT_GPIOTE_INST(DT_ALIAS(sw0), gpios)
#define GPIOTE_NODE	DT_NODELABEL(_CONCAT(gpiote, GPIOTE_INST))
#endif

const struct gpio_dt_spec int1_tilt = GPIO_DT_SPEC_GET(DT_NODELABEL(tilt_int1), gpios);
const struct gpio_dt_spec int1_in = GPIO_DT_SPEC_GET(DT_NODELABEL(in_int1), gpios);
#if (KERNEL_VERSION_MAJOR == 3) && (KERNEL_VERSION_MINOR >= 5)
const nrfx_gpiote_t gpiote = NRFX_GPIOTE_INSTANCE(GPIOTE_INST);
#endif

//nrfx_timer_t timer0 = NRFX_TIMER_INSTANCE(0);
nrfx_timer_t timer1 = NRFX_TIMER_INSTANCE(1);
nrfx_timer_t timer2 = NRFX_TIMER_INSTANCE(2);

uint8_t tilt_channel, in_channel;


void counter_init(nrfx_timer_t *p_timer)
{
    uint32_t base_frequency = NRF_TIMER_BASE_FREQUENCY_GET(p_timer->p_reg);
    nrfx_timer_config_t timer_config = NRFX_TIMER_DEFAULT_CONFIG(base_frequency);

    timer_config.bit_width=NRF_TIMER_BIT_WIDTH_32;
    timer_config.mode = NRF_TIMER_MODE_COUNTER;
    
    nrfx_timer_init(p_timer, &timer_config, NULL);
    nrfx_timer_enable(p_timer);
}


void counter1_init(void)
{
	counter_init(&timer1);
}
void counter2_init(void)
{
	counter_init(&timer2);
}
/*
void timer_init(nrfx_timer_event_handler_t irq_handler)
{    
    uint32_t base_frequency = NRF_TIMER_BASE_FREQUENCY_GET(timer3.p_reg);
    nrfx_timer_config_t timer_config = NRFX_TIMER_DEFAULT_CONFIG(base_frequency);

    timer_config.bit_width=NRF_TIMER_BIT_WIDTH_24;
    nrfx_timer_init(&timer3, &timer_config, irq_handler);    

    uint32_t ticks = nrfx_timer_us_to_ticks(&timer3, 1000000);
    nrfx_timer_extended_compare(&timer3, NRF_TIMER_CC_CHANNEL0, ticks, NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK, true);
    nrfx_timer_compare_int_enable(&timer3,0);
    nrfx_timer_enable(&timer3);
}
*/
void gpiote_init(uint32_t pin_tilt, uint32_t pin_in)
{        
    uint32_t ret; 
#if (KERNEL_VERSION_MAJOR == 3) && (KERNEL_VERSION_MINOR >= 5)    
    (void)nrfx_gpiote_init(&gpiote, NRFX_GPIOTE_DEFAULT_CONFIG_IRQ_PRIORITY);
    ret = nrfx_gpiote_channel_alloc(&gpiote, &tilt_channel);
#else 
    (void)nrfx_gpiote_init(NRFX_GPIOTE_DEFAULT_CONFIG_IRQ_PRIORITY);
    ret = nrfx_gpiote_channel_alloc(&tilt_channel);
#endif

    
    if (ret != NRFX_SUCCESS) {
        //printk("%d %x\n",__LINE__, ret);
    }   
#if (KERNEL_VERSION_MAJOR == 3) && (KERNEL_VERSION_MINOR >= 5)         
    ret = nrfx_gpiote_channel_alloc(&gpiote, &in_channel);
#else
    ret = nrfx_gpiote_channel_alloc(&in_channel);
#endif    
    if (ret != NRFX_SUCCESS) {
        //printk("%d %x\n",__LINE__, ret);
    }    
    static const nrf_gpio_pin_pull_t pull_config = NRF_GPIO_PIN_PULLUP;

	const nrfx_gpiote_trigger_config_t trigger_tilt_config = {
		.trigger = NRFX_GPIOTE_TRIGGER_LOTOHI,
		.p_in_channel = &tilt_channel,
	};

	const nrfx_gpiote_trigger_config_t trigger_in_config = {
		.trigger = NRFX_GPIOTE_TRIGGER_LOTOHI,
		.p_in_channel = &in_channel,
	};    

    static const nrfx_gpiote_handler_config_t handler_config = {
        .handler = NULL,
    };
#if (KERNEL_VERSION_MAJOR == 3) && (KERNEL_VERSION_MINOR >= 5)
    nrfx_gpiote_input_pin_config_t in_input_config = {
        .p_pull_config = &pull_config,
        .p_trigger_config = &trigger_in_config,
        .p_handler_config = &handler_config
    };
    nrfx_gpiote_input_pin_config_t tilt_input_config = {
        .p_pull_config = &pull_config,
        .p_trigger_config = &trigger_tilt_config,
        .p_handler_config = &handler_config
    };
    ret = nrfx_gpiote_input_configure(&gpiote, IN_PIN, &in_input_config);
    if (ret != NRFX_SUCCESS) {
        //printk("%d %x\n",__LINE__, ret);
    }    
    ret = nrfx_gpiote_input_configure(&gpiote,TILT_PIN, &tilt_input_config);
    if (ret != NRFX_SUCCESS) {
        //printk("%d %x\n",__LINE__, ret);
    }
    nrfx_gpiote_trigger_enable(&gpiote, IN_PIN, false);
    nrfx_gpiote_trigger_enable(&gpiote, TILT_PIN, false);
#else

    nrfx_gpiote_input_config_t config = NRFX_GPIOTE_DEFAULT_INPUT_CONFIG;    
    config.pull = NRF_GPIO_PIN_PULLUP;    

    ret = nrfx_gpiote_input_configure(pin_tilt, &config, &trigger_tilt_config,NULL);
    if (ret != NRFX_SUCCESS) {
        //printk("%d %x\n",__LINE__, ret);
    }
    ret = nrfx_gpiote_input_configure(pin_in, &config, &trigger_in_config,NULL);
    if (ret != NRFX_SUCCESS) {
        //printk("%d %x\n",__LINE__, ret);
    }
    nrfx_gpiote_trigger_enable(pin_in, 0);
    nrfx_gpiote_trigger_enable(pin_tilt, 0);
#endif

    
}
/*
void ppi_timer_stop_counter_init(void)
{
   static uint8_t stop1_channel,stop2_channel;
   nrfx_ppi_channel_alloc(&stop1_channel);
   nrfx_ppi_channel_alloc(&stop2_channel); 
   nrfx_ppi_channel_assign(
                stop1_channel,
                nrfx_timer_event_address_get(&timer3,NRF_TIMER_EVENT_COMPARE0),
                nrfx_timer_task_address_get(&timer1,NRF_TIMER_TASK_STOP)
                );
   nrfx_ppi_channel_assign(
                stop2_channel,
                nrfx_timer_event_address_get(&timer3,NRF_TIMER_EVENT_COMPARE0),
                nrfx_timer_task_address_get(&timer2,NRF_TIMER_TASK_STOP)
                );             
   nrfx_ppi_channel_enable(stop1_channel);
   nrfx_ppi_channel_enable(stop2_channel);
}
*/
void ppi_timer_start_counter_init(void) {
    static uint8_t start1_channel,start2_channel;
    uint32_t ret = nrfx_ppi_channel_alloc(&start1_channel);
    if (ret != NRFX_SUCCESS) {
        //printk("%d %x\n",__LINE__, ret);
    }      
    ret = nrfx_ppi_channel_alloc(&start2_channel); 
    if (ret != NRFX_SUCCESS) {
        //printk("%d %x\n",__LINE__, ret);
    }      
    ret = nrfx_ppi_channel_assign(
        start1_channel,
        nrfx_rtc_event_address_get(&rtc, NRF_RTC_EVENT_COMPARE_0),        
        nrfx_timer_task_address_get(&timer1,NRF_TIMER_TASK_START)
    );
    if (ret != NRFX_SUCCESS) {
        //printk("%d %x\n",__LINE__, ret);
    }      
    ret = nrfx_ppi_channel_assign(
        start2_channel,
        nrfx_rtc_event_address_get(&rtc, NRF_RTC_EVENT_COMPARE_0),
        nrfx_timer_task_address_get(&timer2,NRF_TIMER_TASK_START)
    );
    if (ret != NRFX_SUCCESS) {
        //printk("%d %x\n",__LINE__, ret);
    }      
    ret = nrfx_ppi_channel_enable(start1_channel);
    if (ret != NRFX_SUCCESS) {
        //printk("%d %x\n",__LINE__, ret);
    }     
    ret = nrfx_ppi_channel_enable(start2_channel);
    if (ret != NRFX_SUCCESS) {
        //printk("%d %x\n",__LINE__, ret);
    }     
}

void ppi_gpiote_counter_init(void)
{
    /**/
    static uint8_t count1_channel,count2_channel;
    uint32_t ret = nrfx_ppi_channel_alloc(&count1_channel);
    if (ret != NRFX_SUCCESS) {
        //printk("%d %x\n",__LINE__, ret);
    }     
    ret = nrfx_ppi_channel_alloc(&count2_channel);    
    if (ret != NRFX_SUCCESS) {
        //printk("%d %x\n",__LINE__, ret);
    } 
#if (KERNEL_VERSION_MAJOR == 3) && (KERNEL_VERSION_MINOR >= 5)
    ret = nrfx_ppi_channel_assign(
        count1_channel,
        nrfx_gpiote_in_event_address_get(&gpiote, int1_tilt.pin),      
        nrfx_timer_task_address_get(&timer1,NRF_TIMER_TASK_COUNT)
    );
    if (ret != NRFX_SUCCESS) {
        //printk("%d %x\n",__LINE__, ret);
    }     
    ret = nrfx_ppi_channel_assign(
        count2_channel,
        nrfx_gpiote_in_event_address_get(&gpiote, int1_in.pin),
        nrfx_timer_task_address_get(&timer2,NRF_TIMER_TASK_COUNT)
    );
#else
    ret = nrfx_ppi_channel_assign(
        count1_channel,
        nrfx_gpiote_in_event_address_get(int1_tilt.pin),      
        nrfx_timer_task_address_get(&timer1,NRF_TIMER_TASK_COUNT)
    );
    if (ret != NRFX_SUCCESS) {
    }     
    ret = nrfx_ppi_channel_assign(
        count2_channel,
        nrfx_gpiote_in_event_address_get(int1_in.pin),
        nrfx_timer_task_address_get(&timer2,NRF_TIMER_TASK_COUNT)
    );

#endif
    if (ret != NRFX_SUCCESS) {
        //printk("%d %x\n",__LINE__, ret);
    }     
    ret = nrfx_ppi_channel_enable(count1_channel);
    if (ret != NRFX_SUCCESS) {
        //printk("%d %x\n",__LINE__, ret);
    }

    ret = nrfx_ppi_channel_enable(count2_channel);
    if (ret != NRFX_SUCCESS) {
        //printk("%d %x\n",__LINE__, ret);
    }     
}
